@@ -15,7 +15,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: io.h,v 1.71 2006/03/21 16:13:47 essej Exp $
 */
 
 #ifndef __ardour_io_h__
@@ -24,20 +23,21 @@
 #include <string>
 #include <vector>
 #include <cmath>
-#include <sigc++/signal_system.h>
+#include <sigc++/signal.h>
 #include <jack/jack.h>
 
-#include <pbd/lockmonitor.h>
+#include <glibmm/thread.h>
+
 #include <pbd/fastlog.h>
 #include <pbd/undo.h>
-#include <pbd/atomic.h>
-#include <midi++/controllable.h>
+#include <pbd/statefuldestructible.h> 
+#include <pbd/controllable.h>
 
 #include <ardour/ardour.h>
-#include <ardour/stateful.h>
 #include <ardour/utils.h>
-#include <ardour/state_manager.h>
 #include <ardour/curve.h>
+#include <ardour/types.h>
+#include <ardour/data_type.h>
 
 using std::string;
 using std::vector;
@@ -52,7 +52,12 @@ class Port;
 class Connection;
 class Panner;
 
-class IO : public Stateful, public ARDOUR::StateManager
+/** A collection of input and output ports with connections.
+ *
+ * An IO can contain ports of varying types, making routes/inserts/etc with
+ * varied combinations of types (eg MIDI and audio) possible.
+ */
+class IO : public PBD::StatefulDestructible
 {
 
   public:
@@ -60,7 +65,10 @@ class IO : public Stateful, public ARDOUR::StateManager
 
 	IO (Session&, string name, 
 	    int input_min = -1, int input_max = -1, 
-	    int output_min = -1, int output_max = -1);
+	    int output_min = -1, int output_max = -1,
+	    DataType default_type = DataType::AUDIO);
+
+	IO (Session&, const XMLNode&, DataType default_type = DataType::AUDIO);
 
 	virtual ~IO();
 
@@ -74,29 +82,34 @@ class IO : public Stateful, public ARDOUR::StateManager
 	void set_output_minimum (int n);
 	void set_output_maximum (int n);
 
+	DataType default_type() const { return _default_type; }
+
 	const string& name() const { return _name; }
 	virtual int set_name (string str, void *src);
 	
-	virtual void silence  (jack_nframes_t, jack_nframes_t offset);
+	virtual void silence  (nframes_t, nframes_t offset);
 
-	void pan (vector<Sample*>& bufs, uint32_t nbufs, jack_nframes_t nframes, jack_nframes_t offset, gain_t gain_coeff);
-	void pan_automated (vector<Sample*>& bufs, uint32_t nbufs, jack_nframes_t start_frame, jack_nframes_t end_frame, 
-			    jack_nframes_t nframes, jack_nframes_t offset);
-	void collect_input  (vector<Sample*>&, uint32_t nbufs, jack_nframes_t nframes, jack_nframes_t offset);
-	void deliver_output (vector<Sample *>&, uint32_t nbufs, jack_nframes_t nframes, jack_nframes_t offset);
-	void deliver_output_no_pan (vector<Sample *>&, uint32_t nbufs, jack_nframes_t nframes, jack_nframes_t offset);
-	void just_meter_input (jack_nframes_t start_frame, jack_nframes_t end_frame, 
-			       jack_nframes_t nframes, jack_nframes_t offset);
+	// These should be moved in to a separate object that manipulates an IO
+	
+	void pan (vector<Sample*>& bufs, uint32_t nbufs, nframes_t nframes, nframes_t offset, gain_t gain_coeff);
+	void pan_automated (vector<Sample*>& bufs, uint32_t nbufs, nframes_t start_frame, nframes_t end_frame, 
+			    nframes_t nframes, nframes_t offset);
+	void collect_input  (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, nframes_t offset);
+	void deliver_output (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, nframes_t offset);
+	void deliver_output_no_pan (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, nframes_t offset);
+	void just_meter_input (nframes_t start_frame, nframes_t end_frame, 
+			       nframes_t nframes, nframes_t offset);
 
 	virtual uint32_t n_process_buffers () { return 0; }
 
 	virtual void   set_gain (gain_t g, void *src);
-	void   inc_gain (gain_t delta, void *src);
-	gain_t         gain () const                      { return _desired_gain; }
+	void           inc_gain (gain_t delta, void *src);
+	gain_t         gain () const { return _desired_gain; }
 	virtual gain_t effective_gain () const;
 
 	Panner& panner() { return *_panner; }
-
+	const Panner& panner() const { return *_panner; }
+	
 	int ensure_io (uint32_t, uint32_t, bool clear, void *src);
 
 	int use_input_connection (Connection&, void *src);
@@ -105,8 +118,8 @@ class IO : public Stateful, public ARDOUR::StateManager
 	Connection *input_connection() const { return _input_connection; }
 	Connection *output_connection() const { return _output_connection; }
 
-	int add_input_port (string source, void *src);
-	int add_output_port (string destination, void *src);
+	int add_input_port (string source, void *src, DataType type = DataType::NIL);
+	int add_output_port (string destination, void *src, DataType type = DataType::NIL);
 
 	int remove_input_port (Port *, void *src);
 	int remove_output_port (Port *, void *src);
@@ -122,9 +135,9 @@ class IO : public Stateful, public ARDOUR::StateManager
 	int disconnect_inputs (void *src);
 	int disconnect_outputs (void *src);
 
-	jack_nframes_t output_latency() const;
-	jack_nframes_t input_latency() const;
-	void           set_port_latency (jack_nframes_t);
+	nframes_t output_latency() const;
+	nframes_t input_latency() const;
+	void           set_port_latency (nframes_t);
 
 	Port *output (uint32_t n) const {
 		if (n < _noutputs) {
@@ -145,75 +158,78 @@ class IO : public Stateful, public ARDOUR::StateManager
 	uint32_t n_inputs () const { return _ninputs; }
 	uint32_t n_outputs () const { return _noutputs; }
 
-	SigC::Signal2<void,IOChange,void*> input_changed;
-	SigC::Signal2<void,IOChange,void*> output_changed;
+	sigc::signal<void,IOChange,void*> input_changed;
+	sigc::signal<void,IOChange,void*> output_changed;
 
-	SigC::Signal1<void,void*> gain_changed;
-	SigC::Signal1<void,void*> name_changed;
+	sigc::signal<void,void*> gain_changed;
+	sigc::signal<void,void*> name_changed;
 
 	virtual XMLNode& state (bool full);
 	XMLNode& get_state (void);
 	int set_state (const XMLNode&);
 
-	virtual UndoAction get_memento() const;
-
-
 	static int  disable_connecting (void);
-
 	static int  enable_connecting (void);
-
 	static int  disable_ports (void);
-
 	static int  enable_ports (void);
-
 	static int  disable_panners (void);
-
 	static int  reset_panners (void);
 	
-	static SigC::Signal0<int> PortsLegal;
-	static SigC::Signal0<int> PannersLegal;
-	static SigC::Signal0<int> ConnectingLegal;
-	static SigC::Signal1<void,uint32_t> MoreOutputs;
-	static SigC::Signal0<int> PortsCreated;
+	static sigc::signal<int> PortsLegal;
+	static sigc::signal<int> PannersLegal;
+	static sigc::signal<int> ConnectingLegal;
+	static sigc::signal<void,uint32_t> MoreOutputs;
+	static sigc::signal<int> PortsCreated;
 
-	/* MIDI control */
-
-	void set_midi_to_gain_function (gain_t (*function)(double val)) {
-		_midi_gain_control.midi_to_gain = function;
+	PBD::Controllable& gain_control() {
+		return _gain_control;
 	}
 
-	void set_gain_to_midi_function (double (*function)(gain_t gain)) {
-		_midi_gain_control.gain_to_midi = function;
-	}
-
-	MIDI::Controllable& midi_gain_control() {
-		return _midi_gain_control;
-	}
-
-	virtual void reset_midi_control (MIDI::Port*, bool on);
-
-	virtual void send_all_midi_feedback ();
-	virtual MIDI::byte* write_midi_feedback (MIDI::byte*, int32_t& bufsize);
-	
 	/* Peak metering */
 
 	float peak_input_power (uint32_t n) { 
-		if (n < std::max(_ninputs, _noutputs)) {
-			float x = _stored_peak_power[n];
-			if(x > 0.0) {
-				return 20 * fast_log10(x);
-			} else {
-				return minus_infinity();
-			}
+		if (_ninputs == 0) {
+			return minus_infinity();
+		} else if (n < std::max (_ninputs, _noutputs)) {
+			return _visible_peak_power[n];
 		} else {
 			return minus_infinity();
 		}
 	}
 
-	static SigC::Signal0<void> GrabPeakPower;
+	float max_peak_power (uint32_t n) {
+		if (_ninputs == 0) {
+			return minus_infinity();
+		} else if (n < std::max (_ninputs, _noutputs)) {
+			return _max_peak_power[n];
+		} else {
+			return minus_infinity();
+		}
+	}
+
+	void reset_max_peak_meters ();
+
+	
+    static void update_meters();
+
+  private: 
+
+    static sigc::signal<void>   Meter;
+    static Glib::StaticMutex    m_meter_signal_lock;
+    sigc::connection            m_meter_connection;
+
+  public:
 
 	/* automation */
 
+	static void set_automation_interval (jack_nframes_t frames) {
+		_automation_interval = frames;
+	}
+
+	static jack_nframes_t automation_interval() { 
+		return _automation_interval;
+	}
+
 	void clear_automation ();
 
 	bool gain_automation_recording() const { 
@@ -228,22 +244,14 @@ class IO : public Stateful, public ARDOUR::StateManager
 
 	virtual void set_gain_automation_state (AutoState);
 	AutoState gain_automation_state() const { return _gain_automation_curve.automation_state(); }
-	SigC::Signal0<void> gain_automation_state_changed;
+	sigc::signal<void> gain_automation_state_changed;
 
 	virtual void set_gain_automation_style (AutoStyle);
 	AutoStyle gain_automation_style () const { return _gain_automation_curve.automation_style(); }
-	SigC::Signal0<void> gain_automation_style_changed;
+	sigc::signal<void> gain_automation_style_changed;
 
-	static void set_automation_interval (jack_nframes_t frames) {
-		_automation_interval = frames;
-	}
-
-	static jack_nframes_t automation_interval() { 
-		return _automation_interval;
-	}
-
-	virtual void transport_stopped (jack_nframes_t now);
-	virtual void automation_snapshot (jack_nframes_t now);
+	virtual void transport_stopped (nframes_t now);
+	void automation_snapshot (nframes_t now);
 
 	ARDOUR::Curve& gain_automation_curve () { return _gain_automation_curve; }
 
@@ -253,8 +261,6 @@ class IO : public Stateful, public ARDOUR::StateManager
 	void start_pan_touch (uint32_t which);
 	void end_pan_touch (uint32_t which);
 
-	id_t id() const { return _id; }
-
 	void defer_pan_reset ();
 	void allow_pan_reset ();
 
@@ -266,7 +272,7 @@ class IO : public Stateful, public ARDOUR::StateManager
 	int ports_became_legal ();
 
   private:
-	mutable PBD::Lock io_lock;
+	mutable Glib::Mutex io_lock;
 
   protected:
 	Session&            _session;
@@ -274,18 +280,20 @@ class IO : public Stateful, public ARDOUR::StateManager
 	gain_t              _gain;
 	gain_t              _effective_gain;
 	gain_t              _desired_gain;
-	PBD::NonBlockingLock declick_lock;
+	Glib::Mutex         declick_lock;
 	vector<Port*>       _outputs;
 	vector<Port*>       _inputs;
 	vector<float>       _peak_power;
-	vector<float>       _stored_peak_power;
+	vector<float>       _visible_peak_power;
+	vector<float>       _max_peak_power;
 	string              _name;
 	Connection*         _input_connection;
 	Connection*         _output_connection;
-	id_t                _id;
 	bool                 no_panner_reset;
-	bool                 deliver_skip_gain;
 	XMLNode*             deferred_state;
+	DataType            _default_type;
+	bool                _ignore_gain_on_deliver;
+	
 
 	virtual void set_deferred_state() {}
 
@@ -294,50 +302,34 @@ class IO : public Stateful, public ARDOUR::StateManager
 
 	virtual uint32_t pans_required() const { return _ninputs; }
 
-	static void apply_declick (vector<Sample*>&, uint32_t nbufs, jack_nframes_t nframes, 
+	static void apply_declick (vector<Sample*>&, uint32_t nbufs, nframes_t nframes, 
 				   gain_t initial, gain_t target, bool invert_polarity);
 
-	struct MIDIGainControl : public MIDI::Controllable {
-	    MIDIGainControl (IO&, MIDI::Port *);
-	    void set_value (float);
-
-	    void send_feedback (gain_t);
-	    MIDI::byte* write_feedback (MIDI::byte* buf, int32_t& bufsize, gain_t val, bool force = false);
-
+	struct GainControllable : public PBD::Controllable {
+	    GainControllable (std::string name, IO& i) : Controllable (name), io (i) {}
+	 
+	    void set_value (float val);
+	    float get_value (void) const;
+   
 	    IO& io;
-	    bool setting;
-	    MIDI::byte last_written;
-
-	    gain_t (*midi_to_gain) (double val);
-	    double (*gain_to_midi) (gain_t gain);
 	};
 
-	MIDIGainControl _midi_gain_control;
+	GainControllable _gain_control;
 
-	/* state management */
+	nframes_t last_automation_snapshot;
+	static nframes_t _automation_interval;
 
-	Change               restore_state (State&);
-	StateManager::State* state_factory (std::string why) const;
-	void                 send_state_changed();
-
-	bool get_midi_node_info (XMLNode * node, MIDI::eventType & ev, MIDI::channel_t & chan, MIDI::byte & additional);
-	bool set_midi_node_info (XMLNode * node, MIDI::eventType ev, MIDI::channel_t chan, MIDI::byte additional);
-	
-	/* automation */
-
-	jack_nframes_t last_automation_snapshot;
-	static jack_nframes_t _automation_interval;
-
-        AutoState      _gain_automation_state;
+	AutoState      _gain_automation_state;
 	AutoStyle      _gain_automation_style;
 
 	bool     apply_gain_automation;
 	Curve    _gain_automation_curve;
 	
-	int  save_automation (const string&);
-	int  load_automation (const string&);
-	
-	PBD::NonBlockingLock automation_lock;
+	Glib::Mutex automation_lock;
+
+	virtual int set_automation_state (const XMLNode&);
+	virtual XMLNode& get_automation_state ();
+	virtual int load_automation (std::string path);
 
 	/* AudioTrack::deprecated_use_diskstream_connections() needs these */
 
@@ -354,18 +346,18 @@ class IO : public Stateful, public ARDOUR::StateManager
 
 	/* are these the best variable names ever, or what? */
 
-	SigC::Connection input_connection_configuration_connection;
-	SigC::Connection output_connection_configuration_connection;
-	SigC::Connection input_connection_connection_connection;
-	SigC::Connection output_connection_connection_connection;
+	sigc::connection input_connection_configuration_connection;
+	sigc::connection output_connection_configuration_connection;
+	sigc::connection input_connection_connection_connection;
+	sigc::connection output_connection_connection_connection;
 
 	static bool panners_legal;
 	
 	int connecting_became_legal ();
 	int panners_became_legal ();
-	SigC::Connection connection_legal_c;
-	SigC::Connection port_legal_c;
-	SigC::Connection panner_legal_c;
+	sigc::connection connection_legal_c;
+	sigc::connection port_legal_c;
+	sigc::connection panner_legal_c;
 
 	int _input_minimum;
 	int _input_maximum;
@@ -395,7 +387,7 @@ class IO : public Stateful, public ARDOUR::StateManager
 	int make_connections (const XMLNode&);
 
 	void setup_peak_meters ();
-	void grab_peak_power ();
+	void meter ();
 
 	bool ensure_inputs_locked (uint32_t, bool clear, void *src);
 	bool ensure_outputs_locked (uint32_t, bool clear, void *src);
@@ -404,6 +396,6 @@ class IO : public Stateful, public ARDOUR::StateManager
 	int32_t find_output_port_hole ();
 };
 
-}; /* namespace ARDOUR */
+} // namespace ARDOUR
 
 #endif /*__ardour_io_h__ */
