@@ -15,7 +15,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: session_events.cc,v 1.33 2005/12/28 21:38:53 essej Exp $
 */
 
 #include <cmath>
@@ -24,16 +23,16 @@
 #include <ardour/timestamps.h>
 
 #include <pbd/error.h>
-#include <pbd/lockmonitor.h>
+#include <glibmm/thread.h>
 
 #include <ardour/ardour.h>
 #include <ardour/session.h>
-#include <ardour/diskstream.h>
+#include <ardour/audio_diskstream.h>
 
 #include "i18n.h"
 
 using namespace ARDOUR;
-using namespace SigC;
+using namespace PBD;
 
 MultiAllocSingleReleasePool Session::Event::pool ("event", sizeof (Session::Event), 512);
 
@@ -59,21 +58,21 @@ static const char* event_names[] = {
 };
 
 void
-Session::add_event (jack_nframes_t frame, Event::Type type, jack_nframes_t target_frame)
+Session::add_event (nframes_t frame, Event::Type type, nframes_t target_frame)
 {
 	Event* ev = new Event (type, Event::Add, frame, target_frame, 0);
 	queue_event (ev);
 }
 
 void
-Session::remove_event (jack_nframes_t frame, Event::Type type)
+Session::remove_event (nframes_t frame, Event::Type type)
 {
 	Event* ev = new Event (type, Event::Remove, frame, 0, 0);
 	queue_event (ev);
 }
 
 void
-Session::replace_event (Event::Type type, jack_nframes_t frame, jack_nframes_t target)
+Session::replace_event (Event::Type type, nframes_t frame, nframes_t target)
 {
 	Event* ev = new Event (type, Event::Replace, frame, target, 0);
 	queue_event (ev);
@@ -158,7 +157,7 @@ Session::merge_event (Event* ev)
 	default:
 		for (Events::iterator i = events.begin(); i != events.end(); ++i) {
 			if ((*i)->type == ev->type && (*i)->action_frame == ev->action_frame) {
-				error << compose(_("Session: cannot have two events of type %1 at the same frame (%2)."), 
+			  error << string_compose(_("Session: cannot have two events of type %1 at the same frame (%2)."), 
 						 event_names[ev->type], ev->action_frame) << endmsg;
 				return;
 			}
@@ -305,14 +304,27 @@ Session::process_event (Event* ev)
 	*/
 
 	if (non_realtime_work_pending()) {
-		immediate_events.insert (immediate_events.end(), ev);
-		_remove_event (ev);
-		return;
+		
+		/* except locates, which we have the capability to handle */
+
+		if (ev->type != Event::Locate) {
+			immediate_events.insert (immediate_events.end(), ev);
+			_remove_event (ev);
+			return;
+		}
 	}
 
 	switch (ev->type) {
 	case Event::SetLoop:
-		set_auto_loop (ev->yes_or_no);
+		set_play_loop (ev->yes_or_no);
+		break;
+
+	case Event::AutoLoop:
+		if (play_loop) {
+			start_locate (ev->target_frame, true, false, Config->get_seamless_loop());
+		}
+		remove = false;
+		del = false;
 		break;
 
 	case Event::Locate:
@@ -341,7 +353,7 @@ Session::process_event (Event* ev)
 		
 	case Event::PunchIn:
 		// cerr << "PunchIN at " << transport_frame() << endl;
-		if (punch_in && record_status() == Enabled) {
+		if (Config->get_punch_in() && record_status() == Enabled) {
 			enable_record ();
 		}
 		remove = false;
@@ -350,7 +362,7 @@ Session::process_event (Event* ev)
 		
 	case Event::PunchOut:
 		// cerr << "PunchOUT at " << transport_frame() << endl;
-		if (punch_out) {
+		if (Config->get_punch_out()) {
 			step_back_from_record ();
 		}
 		remove = false;
@@ -380,28 +392,20 @@ Session::process_event (Event* ev)
 		del = false;
 		break;
 
-	case Event::AutoLoop:
-		if (auto_loop) {
-			start_locate (ev->target_frame, true, false, seamless_loop);
-		}
-		remove = false;
-		del = false;
-		break;
-
 	case Event::Overwrite:
-		overwrite_some_buffers (static_cast<DiskStream*>(ev->ptr));
+		overwrite_some_buffers (static_cast<AudioDiskstream*>(ev->ptr));
 		break;
 
 	case Event::SetDiskstreamSpeed:
-		set_diskstream_speed (static_cast<DiskStream*> (ev->ptr), ev->speed);
+		set_diskstream_speed (static_cast<AudioDiskstream*> (ev->ptr), ev->speed);
 		break;
 
 	case Event::SetSlaveSource:
-		set_slave_source (ev->slave, ev->target_frame);
+		set_slave_source (ev->slave);
 		break;
 
 	case Event::Audition:
-		set_audition (static_cast<AudioRegion*> (ev->ptr));
+		set_audition (ev->region);
 		break;
 
 	case Event::InputConfigurationChange:
@@ -419,7 +423,7 @@ Session::process_event (Event* ev)
 		break;
 
 	default:
-		fatal << compose(_("Programming error: illegal event type in process_event (%1)"), ev->type) << endmsg;
+	  fatal << string_compose(_("Programming error: illegal event type in process_event (%1)"), ev->type) << endmsg;
 		/*NOTREACHED*/
 		break;
 	};
